
/*****************************************************************************
 *                                                                           *
 *  PMB source code, MPI-1 part                           ---/---            *
 *                                                          / /              *
 *  Copyright (c) PALLAS GmbH 1998                         / / /             *
 *                                                        / / / /            *
 *                                                         / / /             *
 *  Authors: Hans-Joachim Plum , Gero Ritzenhoefer          / / PALLAS       *
 *  Revision: 2.0                                         ---/---            *
 *  Date: 1998/03/10                                                         *
 *                                                                           *
 *****************************************************************************/
/*-----------------------------------------------------------
Functions implemented in this module: 

void PingPong(struct comm_info* c_info,int size,int n_sample,double* time)

------------------------------------------------------------*/
#include "declare.h"

/*************************************************************************/
void PingPong(struct comm_info* c_info,int size,int n_sample,double* time)
/*************************************************************************/

/*------------------------------------------------------------
             VARIABLE |       TYPE        |   MEANING
------------------------------------------------------------
Input      : c_info   | struct comm_info* | see comm_info.h 
             size     | int               | message length in byte
	     n_sample | int               | repetition count
                      |                   |
Output     : time     | double*           | *time: time/sample in usec
                      |                   |
In/Out     :  -       | -                 | -
                      |                   |  
------------------------------------------------------------
------------------------------------------------------------
Description: 
            Two processes take part. A message is send  (MPI_Send) 
	    then receive (MPI_Recv) and rebounced.
	    
SCHEMATIC VIEW:  __         
                |p1|\
	        |__| \  __ 
		      \|p2|
		      /|__|
	         __  / 	  
                |p1|/
	        |__|
-------------------------------------------------------------*/
{
  double t1, t2;
  int    i, ierr=0;
  
  Type_Size s_size,r_size;
  int s_num, r_num;
  int s_tag, r_tag;
  int dest, source;
  MPI_Status stat;

  /*  GET SIZE OF DATA TYPE */  
  MPI_Type_size(c_info->s_data_type,&s_size);
  MPI_Type_size(c_info->r_data_type,&r_size);
  if ((s_size!=0) && (r_size!=0))
    {
      s_num=size/s_size;
      r_num=size/r_size;
    } 
  s_tag = 1;
  r_tag = c_info->select_tag ? s_tag : MPI_ANY_TAG;
  
  if (c_info->rank == c_info->pair0)
    {
      /*  CALCULATE SOURCE AND DESTINATION */ 
      dest = c_info->pair1;
      source = c_info->select_source ? dest : MPI_ANY_SOURCE;
      
      t1 = MPI_Wtime();
      for(i=0;i< n_sample;i++)
	{
	  ierr = MPI_Send(c_info->s_buffer,s_num,c_info->s_data_type,dest,
			  s_tag,c_info->communicator);
	  MPI_ERRHAND(ierr);
	  ierr = MPI_Recv(c_info->r_buffer,r_num,c_info->r_data_type,source,
			  r_tag,c_info->communicator,&stat);
	  MPI_ERRHAND(ierr);
	}
      t2 = MPI_Wtime();
      *time=(t2 - t1)/n_sample;
    }
  else if (c_info->rank == c_info->pair1)
    {
      dest =c_info->pair0 ;
      source = c_info->select_source ? dest : MPI_ANY_SOURCE;
      
      t1 = MPI_Wtime();
      for(i=0 ;i< n_sample;i++)
	{
	  ierr= MPI_Recv(c_info->r_buffer,r_num,c_info->r_data_type,source,
			 r_tag,c_info->communicator,&stat);
	  MPI_ERRHAND(ierr);
	  ierr= MPI_Send(c_info->s_buffer,s_num,c_info->s_data_type,dest,
			 s_tag,c_info->communicator);
	  MPI_ERRHAND(ierr);
	}
     t2 = MPI_Wtime();
      
     *time=(t2 - t1)/n_sample;
    }
  else 
    { 
      *time = 0.;
    }
}


