
/*****************************************************************************
 *                                                                           *
 *  PMB source code, MPI-1 part                           ---/---            *
 *                                                          / /              *
 *  Copyright (c) PALLAS GmbH 1998                         / / /             *
 *                                                        / / / /            *
 *                                                         / / /             *
 *  Authors: Hans-Joachim Plum , Gero Ritzenhoefer          / / PALLAS       *
 *  Revision: 2.0                                         ---/---            *
 *  Date: 1998/03/10                                                         *
 *                                                                           *
 *****************************************************************************/
/*---------------------------------------------------------------------


Functions implemented in this module: 

void Warm_Up  (struct comm_info* c_info,
               struct Bench* Bmark,
               int iter)

Used in modules : pmb.c
---------------------------------------------------------------------*/

#include "declare.h"
#include "Benchmark.h"

/**********************************************************************/
void Warm_Up  (struct comm_info* c_info,
               struct Bench* Bmark,
               int iter)
/**********************************************************************

----------------------------------------------------------------------
             VARIABLE |       TYPE        |   MEANING
----------------------------------------------------------------------
Input      : c_info   | struct comm_info* | see comm_info.h 
             Bmark    | struct Bench*     | Benchmark to run
             iter     | int               | number of iteration in pmb
Output     :          |                   |
                      |                   |
In/Out     :  -       | -                 | -
                      |                   |  
----------------------------------------------------------------------
----------------------------------------------------------------------
Description: "Warm up" of benchmark: 
             Benchmark is run N_WARMUP times (see settinhs.h ) 
             with maximum message length
             N_BARR (see settings.h) Barriers follow
--------------------------------------------------------------------*/
{
  int ibarr;
  if( c_info->rank >= 0 )
    {
      if( iter == 0 )
	{
	  int size = 1<<MAXMSGLOG;
	  double t;
	  int n_sample = N_WARMUP;

	  Bmark->Benchmark(c_info, size, n_sample, &t);
	}
      for(ibarr=0; ibarr<N_BARR; ibarr++)
	MPI_Barrier(c_info->communicator);
    }
}
