
/*****************************************************************************
 *                                                                           *
 *  PMB source code, MPI-1 part                           ---/---            *
 *                                                          / /              *
 *  Copyright (c) PALLAS GmbH 1998                         / / /             *
 *                                                        / / / /            *
 *                                                         / / /             *
 *  Authors: Hans-Joachim Plum , Gero Ritzenhoefer          / / PALLAS       *
 *  Revision: 2.0                                         ---/---            *
 *  Date: 1998/03/10                                                         *
 *                                                                           *
 *****************************************************************************/
/*----------------------------------------------------------
Functions implemented in this module: 

main(int argc, char **argv)

------------------------------------------------------------*/
#include "mpi.h"

#include "declare.h"
#include "Benchmark.h"

/* EXTERN PROTOTYPES */
extern void Basic_Input         (struct comm_info*,struct Bench**,
                                 int*,char***,int* );
extern void Init_pointers       (struct comm_info* c_info);
extern int  Init_communicator   (struct comm_info*,int);
extern void Init_buffers        (struct comm_info* c_info, struct Bench* Bmark,
                                 int size);
int valid                       (struct comm_info* c_info, struct Bench* Bmark, int NP);
extern void Show_selections     (struct comm_info*, struct Bench*);
extern void Warm_Up             (struct comm_info*, struct Bench*, int);
extern void Output              (struct comm_info*,struct Bench*,
                                 int,int,int,int,double*);
extern void Free_ALL            (struct comm_info* c_info, 
                                 struct Bench** P_BList);
extern void Err_Hand            (int,int);

extern void Output_eff_bandwidth();

/**********************************************************************/
main(int argc, char **argv)
/**********************************************************************/
{
struct comm_info C_INFO;   /* BASIC SETTINGS STRUCTURE    */
struct Bench* BList;       /* List of Benchmarks          */

struct Bench* Bmark;

char *p;
int  i,j,iter,ierr;
int  NP,NP_min;
int  do_it  ; 

int   header;              /* FLAG WETHER WARMUP NECESSARY */
int     size;              /* SIZE OF MESSAGE              */
int n_sample;              /* NUMBER OF MESSAGES           */
double  time;              /* TIMER                        */

Type_Size unit_size;       /* SIZE OF REDUCTION MESSAGE    */


if( (ierr=MPI_Init(&argc,&argv))!=MPI_SUCCESS) Err_Hand(1, ierr);

Init_pointers(&C_INFO);

if(C_INFO.w_num_procs%2)
  {

    if(!C_INFO.w_rank) 
       printf("Please use only even PE numbers!\n");

  }
else
  {

Basic_Input(&C_INFO,&BList,&argc,&argv,&NP_min);

Show_selections(&C_INFO,BList);

/* LOOP OVER INDIVIDUAL BENCHMARKS */
j=0;
while( (p=BList[j].name) )
  {

    C_INFO.group_mode = j + 1; 


 if( BList[j].type != BTYPE_INVALID )
  {
    Bmark = BList+j;

    NP=max(1,min(C_INFO.w_num_procs,NP_min));

    if( Bmark->type == SingleTransfer ) NP = min(2,NP);

    do_it = 1;
/* LOOP OVER PROCESS NUMBERS */
    while ( do_it )       
      {
        if ( valid(&C_INFO,Bmark,NP) )
        {

	if(Init_communicator(&C_INFO,NP)!=0) Err_Hand(0,-1);

	/* MINIMAL OUTPUT IF UNIT IS GIVEN */
	if (C_INFO.w_rank==0)
	  {
	    if(unit != stdout) 
	      printf
              ("# Running %s; see file \"%s_%d\" for results\n",p,OUTPUT_FILENAME,C_INFO.w_num_procs);
	  };
	header=1;

	MPI_Type_size(C_INFO.s_data_type,&unit_size);
	MAXMSG=(1<<MAXMSGLOG)/unit_size* unit_size;
	
        iter=0;
	size =0;

/* LOOP OVER MESSAGE LENGTHS */
        while (size < MAXMSG) 	                           
	      {  
		if ( iter == 0 ) 
		  {
		    size     = 0;
		    n_sample = MSGSPERSAMPLE ;
		  }
		else if ( iter == 1 ) 
		  {  
		    size = ((1<<MINMSGLOG)+(unit_size -1))/unit_size* unit_size;
		    n_sample =max(1,min(TOTALSIZE/size,MSGSPERSAMPLE));
		  }
		else
		  { 
		    size     = min(MAXMSG,size+size);
		    n_sample = max(1,min(TOTALSIZE/size,MSGSPERSAMPLE));
		  }

                /* INITIALIZE COMMUNICATION BUFFERS */
                
	        Init_buffers(&C_INFO, Bmark, MAXMSG);

		/* WARM-UP BEFORE ACTUAL BENCHMARK MEASUREMENT  */

		Warm_Up  (&C_INFO,Bmark,iter);                    

		/* RUN THE BENCHMARK MEASUREMENT             */
                
		Bmark->Benchmark(&C_INFO,size,n_sample,&time);

                /* Synchronization, in particular for idle processes
                   which have to wait in a well defined manner */
                MPI_Barrier(MPI_COMM_WORLD);
                     
	  	Output   (&C_INFO,Bmark,NP,header,size,n_sample,&time);  
		
		header = 0;
		iter++;
	      }
        }

	/* CALCULATE THE NUMBER OF PROCESSES FOR NEXT STEP */
	if( NP == C_INFO.w_num_procs  ) {do_it=0;}
	else
	  {
	    NP=min(NP+NP,C_INFO.w_num_procs);
	  }	  
	
      }

  }
  j++;
  }

Output_eff_bandwidth(&C_INFO);

Free_ALL(&C_INFO, &BList);

}

MPI_Finalize();
}
